package Video::PlaybackMachine::TimeLayout;

####
#### Interface: Video::PlaybackMachine::TimeLayout
####
#### $Revision: 1.1 $
####
#### The one-dimensional equivalent of a Java LayoutManager.
#### Tells us the minimum and preferred times that a bit of Java
#### programming can last.
####
###
####

use strict;
use warnings;
use Carp;

############################# Class Constants #############################

############################## Class Methods ##############################

############################# Object Methods ##############################

sub min_time { }

sub preferred_time { }

1;
